# BlankGame by Rob Miles
# Version 1.0 June 2014
#
# Does nothing much, but in a very extendable way...

import pygame
import random
import copy
from Cheese import *
from Maze import *
from Cracker import *

class ChasedByCrackers:

    # Initialise the game
    # Set up Pygame and create all the game objects
    def __init__(self):
        self.isFinished = False
        self.isRunning = False
        self.windowSize = (800, 600)
        self.backgroundColour = (255, 255, 255)
        self.textColour = (255,0,0)
        self.fps = 120
        
        # pre initialise pyGame's audio engine to avoid sound latency issues
        pygame.mixer.pre_init(frequency=44100)
        pygame.init()

        # initialise pyGame's audio engine
        pygame.mixer.init()

        # load the font
        self.font = pygame.font.Font(None, 50)

        # Create the game surface
        self.surface = pygame.display.set_mode(self.windowSize)
        
        #
        # **** TODO ****
        # Insert the title of the game here
        #
        pygame.display.set_caption("Chased by a Cracker")

        #
        # **** TODO ****
        # Create any game elements here and add them to the class
        # The sample game just draws a piece of cheese on a tablecloth
        # The game will fail if the file cheese.png is not available

        # Create the background
        backgroundImage = pygame.image.load("tablecloth.png")
        #backgroundImage = pygame.image.load("maze.png")
        layoutImage = pygame.image.load("maze.png")
        self.gameMaze = Maze(self.windowSize,backgroundImage,layoutImage)
        
        # Create the cheese
        # Place it at 0,0
        # Give it the size of the window for bounding
        # 
        cheeseImage = pygame.image.load("cheese.png")
        self.gameCheese = Cheese((0,0),self.windowSize,(10,10),cheeseImage,50)

        crackerImage = pygame.image.load("cracker.png")
        self.cracker = Cracker((500,500),self.windowSize,(50,50),360,crackerImage,30, self.gameCheese)

    # Turns off Pygame and quits the game
    #
    def quitGame(self):
     self.isFinished = True
     pygame.quit()
     
    # Draws the screen during gameplay        
    def drawGame(self):

        # Don't draw if the game is over
        if self.isFinished:
            return
        
        # Clear off the previous game screen
        self.surface.fill(self.backgroundColour)

        # Draw the game background image
        self.gameMaze.draw(self.surface)

        #
        # **** TODO ****
        # Create a string that contains status information for 
        # the game. This is then drawn at the top of the screen
        status = "Game Status goes here"
        self.displayMessage(status,0)

        #
        # **** TODO ****
        # Draw the game objects on the screen. 
        # For the sample game we just draw the cheese

        self.gameCheese.draw(self.surface)
        self.cracker.draw(self.surface)

        # Enable the display
        pygame.display.flip()

    # Updates the game during gameplay
    def updateGame(self,deltaTime):
        # make a copy of the objects original position
        oldCheesePosition = copy.copy(self.gameCheese.position)
        oldCheeseRect = copy.copy(self.gameCheese.rect)
     
        # Deal with any movement requests from the keyboard
        for e in pygame.event.get():
            if e.type == pygame.KEYDOWN:
                #
                # **** TODO ****
                # Use the keys to start the player movement
                # In the sample game we use the arrow keys
                # to move the cheese around the screen.
                if e.key == pygame.K_UP:
                    self.gameCheese.StartMoveUp()
                if e.key == pygame.K_DOWN:
                    self.gameCheese.StartMoveDown()
                if e.key == pygame.K_LEFT:
                    self.gameCheese.StartMoveLeft()
                if e.key == pygame.K_RIGHT:
                    self.gameCheese.StartMoveRight()
                if e.key == pygame.K_ESCAPE:
                    self.quitGame()
            if e.type == pygame.KEYUP:
                #
                # **** TODO ****
                # Use the keys to stop the player movement
                # In the sample game we use the arrow keys
                # to move the cheese around the screen.
                if e.key == pygame.K_UP:
                    self.gameCheese.StopMoveUp()
                if e.key == pygame.K_DOWN:
                    self.gameCheese.StopMoveDown()
                if e.key == pygame.K_LEFT:
                    self.gameCheese.StopMoveLeft()
                if e.key == pygame.K_RIGHT:
                    self.gameCheese.StopMoveRight()

        #
        # **** TODO ****
        # Ask each of the game objects to update themselves
        # In the sample game this is just the cheese
        self.gameCheese.update(deltaTime)

        # Test for a collision with the maze
        if self.gameMaze.collides(self.gameCheese.rect):
            # Restore the original position and stop the object
            self.gameCheese.position = oldCheesePosition
            self.gameCheese.rect = oldCheeseRect
            self.gameCheese.movementSpeed=[0,0]

        self.cracker.update(deltaTime)

        if self.gameCheese.cheeseDead:
            self.endGame()


    # Called to start the game running
    def startGame(self):
        
        #
        # **** TODO ****
        # Ask each of the game objects to reset themselves
        # In the sample game this is just the cheese
        # Reset the cheese
        self.gameCheese.reset()
        self.cracker.reset()

        # Set the flag to indicate the game is runing
        self.isRunning=True

    # Called to end the game
    def endGame(self):

        # Clear the flag to indicate the game is over
        self.isRunning=False


    # Called to update the game at the start screen
    # Checks to see if g has been pressed and starts the
    # game if it has been
    def updateStartScreen(self):
        for e in pygame.event.get():
            if e.type == pygame.KEYDOWN:
                if e.key == pygame.K_g:
                    self.startGame()
                if e.key == pygame.K_ESCAPE:
                    self.quitGame()

    # Called to display a centred line of text on the screen
    #     message - the string of text to display
    #     yPos - the Y location of the top of the message
    def displayMessage(self, message, yPos):
        textScore = self.font.render(message, 1, self.textColour)
        postionOfText = (self.windowSize[0]/2 - textScore.get_width()/2, yPos)
        self.surface.blit(textScore, postionOfText)

    # Draws the start screen display 
    def drawStartScreen(self):

        # Don't draw if the game is over
        if self.isFinished:
            return
        
        self.surface.fill(self.backgroundColour)

        # Draw the background image 
        self.gameMaze.draw(self.surface)

        #
        # **** TODO ****
        # Draw the welcome message on the screen

        self.displayMessage("Welcome to Chased by a Cracker", 150)
        self.displayMessage("Press G to play", 500)
        pygame.display.flip()

    # runs the game
    def run(self):

        # Make a copy of the Pygame clock for use by the game
        clock = pygame.time.Clock()
        
        # main game loop

        while not self.isFinished:
            # time to complete the last frame (in secs)
            deltaTime = clock.tick(self.fps)/1000.0

            # Update according to the game mode
            if self.isRunning:
                self.updateGame(deltaTime)
                self.drawGame()
            else:
                self.updateStartScreen()
                self.drawStartScreen()

# Create a new instance of the game and run it
g = ChasedByCrackers()
g.run()


        
    
