import pygame
from pygame.locals import *

class Maze:
    
    # Initialise the Maze object
    #    limit - a tuple giving size of the screen
    #    backgroundImage - the image to be drawn
    #    layoutImage - the image to be used to defined the maze layout
    def __init__(self, limit, backgroundImage, layoutImage):
        # Scale and store the two images 
        self.backgroundImage = pygame.transform.scale(backgroundImage, limit)
        self.layoutImage = pygame.transform.scale(layoutImage, limit)

    # Draws the target on the screen at its current position
    def draw(self, surface):
        surface.blit(self.backgroundImage, (0,0))

    # Collision detection routine
    def collides(self, rect):
        # ensure the right and bottom coordinates are within the rectangle
        topleft = [rect.left, rect.top]
        topright = [rect.right-1, rect.top]
        bottomleft = [rect.left, rect.bottom-1]
        bottomright = [rect.right-1, rect.bottom-1]
        
        # test the 4 corners of the rect for a Red Component of zero
        if self.layoutImage.get_at(topleft).r == 0 and \
           self.layoutImage.get_at(topright).r == 0 and \
           self.layoutImage.get_at(bottomleft).r == 0 and \
           self.layoutImage.get_at(bottomright).r == 0 :
            return False
        else:
            return True
                    
    # Gets the name of this sprite
    def getName(self):
        return "maze"

    # Gets the description of this sprite
    def getDescription(self):
        return "A maze that fills the entire screen, and defines a maze for collision detection"

